using System;
using System.Drawing;
using System.Windows.Forms;
using NaGet.Packages.Install;
using NaGet.Packages;
using System.Text;

namespace AppliStation
{
	/// <summary>
	/// Description of PackageInfoForm.
	/// </summary>
	public partial class PackageInfoForm : Form
	{
		private Package pkg;
		
		public PackageInfoForm()
		{
			InitializeComponent();
		}
		
		public Package SelectedObject {
			get { return pkg; }
			set {
				pkg = value;
				updateSelectedObject();
			}
		}
		
		private void updateSelectedObject()
		{
			bool isInstalledPackage = pkg is InstalledPackage;
			
			iconLabel.Image = null;
			iconLabel.Size = Size.Empty;
			if (isInstalledPackage) {
				Icon icon = AppliStation.Util.PackageUtils.GetIconForPackage((InstalledPackage) pkg);
				if (icon != null) {
					iconLabel.Image = icon.ToBitmap();
					iconLabel.Size = icon.Size;
				}
			}
			nameLabel.Text = string.Format("{0} ({1})", pkg.Name, pkg.Version);
			nameLabel.Font = new Font(SystemFonts.DefaultFont.FontFamily, SystemFonts.DefaultFont.Size*1.5f);
			
			summaryTextBox.Text = pkg.Summary;
			
			if (isInstalledPackage) {
				UninstallInformation uninstInfo = ((InstalledPackage) pkg).UninstallInfo;
				
				string instPath = null;
				if (pkg.Type == InstallerType.ARCHIVE) {
					instPath = System.IO.Path.Combine(NaGet.Env.ArchiveProgramFiles, pkg.Name);
				} else if (System.IO.Directory.Exists(uninstInfo.InstallLocation)) {
					instPath = uninstInfo.InstallLocation;
				}
				
				statusLinkLabel.Links.Clear();
				if (string.IsNullOrEmpty(instPath)) {
					statusLinkLabel.Text = "񂪒񋟂Ă܂";
					statusLinkLabel.Enabled = false;
				} else {
					statusLinkLabel.Text = instPath;
					statusLinkLabel.Enabled = true;
					
					addLinkToLinkLabel(statusLinkLabel, uninstInfo.InstallLocation, uninstInfo.InstallLocation);
				}
			}
			urlLinkLabel.Links.Clear();
			if (pkg.Url != null) {
				urlLinkLabel.Text = pkg.Url.Href;
				urlLinkLabel.Links.Add(0, pkg.Url.Href.Length, pkg.Url.Href);
			} else {
				urlLinkLabel.Text = string.Empty;
				urlLinkLabel.Enabled = false;
				urlLabel.Enabled = false;
			}
			tagsTextBox.Text = pkg.Tags;
			setLinkLabelFromLicense(licenseLinkLabel, pkg.License);
			typeValueLabel.Text = pkg.Type.ToString();
			
			tabControl1.TabPages.Clear();
			foreach (Installer inst in pkg.Installer) {
				TabPage page = new TabPage(string.Format("CXg[ {0}", tabControl1.TabCount+1));
				page.Dock = DockStyle.Fill;
				
				InstallerInfoForm instForm = new InstallerInfoForm();
				instForm.SelectedObject = inst;
				instForm.Dock = DockStyle.Fill;
				page.Controls.Add(instForm);
				
				tabControl1.TabPages.Add(page);
			}
		}
		
		private static void setLinkLabelFromLicense(LinkLabel linklabel, string license)
		{
			linklabel.Enabled = true;
			switch (license ?? string.Empty) {
				case "":
					linklabel.Text = "t[\tgEFA ()";
					linklabel.Enabled = false;
					break;
				case "GPL":
					linklabel.Text = "GPL (I[v\[X)";
					addLinkToLinkLabel(linklabel, "GPL", @"http://www.gnu.org/licenses/gpl.ja.html");
					addLinkToLinkLabel(linklabel, "I[v\[X", @"http://creativecommons.org/licenses/GPL/2.0/deed.ja");
					break;
				case "GPLv2":
					linklabel.Text = "GPLv2 (I[v\[X)";
					addLinkToLinkLabel(linklabel, "GPLv2", @"http://www.gnu.org/licenses/gpl-2.0.html");
					break;
				case "GPLv3":
					linklabel.Text = "GPLv3 (I[v\[X)";
					addLinkToLinkLabel(linklabel, "GPLv3", @"http://www.gnu.org/licenses/gpl-3.0.html");
					break;
				case "LGPL":
					linklabel.Text = "LGPL (I[v\[X)";
					addLinkToLinkLabel(linklabel, "LGPL", @"http://www.gnu.org/licenses/lgpl.ja.html");
					addLinkToLinkLabel(linklabel, "I[v\[X", @"http://creativecommons.org/licenses/LGPL/2.1/deed.ja");
					break;
				case "LGPLv2":
					linklabel.Text = "LGPLv2 (I[v\[X)";
					addLinkToLinkLabel(linklabel, "LGPLv2", @"http://www.gnu.org/licenses/lgpl-2.1.html");
					break;
				case "LGPLv3":
					linklabel.Text = "LGPLv3 (I[v\[X)";
					addLinkToLinkLabel(linklabel, "LGPLv3", @"http://www.gnu.org/licenses/lgpl-3.0.html");
					break;
				case "MPL":
					linklabel.Text = "MPL (I[v\[X)";
					addLinkToLinkLabel(linklabel, "MPL", @"http://www.mozilla.org/MPL/MPL-1.1.html");
					break;
				case "CPL":
					linklabel.Text = "CPL (I[v\[X)";
					linklabel.Links.Add(0, linklabel.Text.Length, @"http://sourceforge.jp/projects/opensource/wiki/licenses%2FCommon_Public_License");
					break;
				case "MIT":
					linklabel.Text = "MITCZX (I[v\[X)";
					addLinkToLinkLabel(linklabel, "MITCZX", @"http://www.opensource.org/licenses/mit-license.php");
					break;
				case "BSD":
					linklabel.Text = "CBSDCZX (I[v\[X)";
					addLinkToLinkLabel(linklabel, "BSDCZX", @"http://www.opensource.org/licenses/bsd-license.php");
					break;
				case "NYSL":
					linklabel.Text = "ςȂĂȂDɂ냉CZX";
					linklabel.Links.Add(0, linklabel.Text.Length, @"http://www.kmonos.net/nysl/");
					break;
				case "Public Domain":
					linklabel.Text = "pubNhC (쌠Kp)";
					linklabel.Links.Add(0, linklabel.Text.Length, @"http://e-words.jp/w/E38391E38396E383AAE38383E382AFE38389E383A1E382A4E383B3.html");
					break;
				default:
					if (license.StartsWith("http://")) {
						linklabel.Text = license;
						linklabel.Links.Add(0, license.Length, license);
					} else {
						linklabel.Text = license;
						linklabel.Links.Add(0, 0, string.Empty);
					}
					break;
			}
			
		}
		
		/// <summary>
		/// Nx̃N}֗\bh
		/// </summary>
		/// <param name="linkLabel">ΏۃNx</param>
		/// <param name="word">NΏە</param>
		/// <param name="link">NIuWFNg</param>
		private static void addLinkToLinkLabel(LinkLabel linkLabel, string word, string link)
		{
			int pos = linkLabel.Text.IndexOf(word, 0);
			if (pos >= 0) {
				linkLabel.Links.Add(pos, word.Length, link);
			} else throw new ArgumentException(string.Format("label text \"{0}\" does not contain \"{1}\"!", linkLabel.Text, word));
		}
		
		private void linkLabelLinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
		{
			string linkData = e.Link.LinkData.ToString();
			
			if (System.IO.Directory.Exists(linkData)) {
				e.Link.Visited = true;
				System.Diagnostics.Process.Start(e.Link.LinkData.ToString());
			} else if (System.Text.RegularExpressions.Regex.IsMatch(linkData, @"(https?|file)://")) {
				e.Link.Visited = true;
				System.Diagnostics.Process.Start(linkData);
			}
		}
	}
}
